<!DOCTYPE HTML>
<html lang="en">
<head>
	<meta charset="utf-8">
	<meta http-equiv="X-UA-Compatible" content="IE=edge">
	<meta name="viewport" content="width=device-width, initial-scale=1">
	<title>OmniConverter Admin Panel</title>
	<!--<link rel="stylesheet" href="http://maxcdn.bootstrapcdn.com/bootstrap/3.2.0/css/bootstrap.min.css">-->
	<link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootswatch/3.3.7/cerulean/bootstrap.min.css">
	<link rel="stylesheet" href="http://maxcdn.bootstrapcdn.com/font-awesome/4.2.0/css/font-awesome.min.css">
	<script src="http://code.jquery.com/jquery.js"></script>
    {#
	<script src="http://maxcdn.bootstrapcdn.com/bootstrap/3.2.0/js/bootstrap.min.js"></script>
    #}
</head>
<body>

<nav class="navbar navbar-default">
  <div class="container">
    <!-- Brand and toggle get grouped for better mobile display -->
    <div class="navbar-header">
      <a class="navbar-brand" href="/admin/">OmniC</a>
    </div>

    <ul class="nav navbar-nav">
        <li class="{% if is_conversion %}active{% endif %}"><a href="/admin/conversion/">Conversion</a></li>
        <li class="{% if is_graph %}active{% endif %}"><a href="/admin/graph/">Graph Explorer</a></li>
    </ul>
  </div><!-- /.container-fluid -->
</nav>

{% block body %}
{% endblock body %}

</body>
</html>
